@@ -3,8 +3,7 @@
 # Python Modules
 import dataclasses
 import json
-from typing import Dict, List, Literal, Optional
-from h11 import Response
+from typing import Any, Dict, List, Literal, Optional
 
 # 3rd Party Modules
 import httpx
@@ -26,6 +25,7 @@ Protocol = Literal["http", "https"]
 
 
 drop_none_values = lambda d: {k: v for k, v in d.items() if v is not None}
+
 BATCH_LIMIT = 500
 
 
@@ -84,8 +84,10 @@ def get_orgs(
     after,
     limit,
     schema,
-) -> Response:
+    timeout,
+) -> httpx.Response:
     """Get orgs."""
+
     res = httpx.get(
         f"{url}/orgs",
         headers=drop_none_values(
@@ -103,6 +105,7 @@ def get_orgs(
                 "schema": schema,
             }
         ),
+        timeout=timeout,
     )
 
     res.raise_for_status()
@@ -156,6 +159,7 @@ class GraphAPI:  # pylint: disable=too-many-instance-attributes
         force_refresh: bool = False,
         schema: Optional[ProductSchemaName] = None,
         filtering: Optional[List[Dict]] = None,
+        timeout: Optional[int] = None,
     ):
         """Get products.
 
@@ -173,6 +177,7 @@ class GraphAPI:  # pylint: disable=too-many-instance-attributes
             schema: Response schema.
             filtering: Filter products.
                 Example: `[{"field":"id","operator":"IN","value":["CCCQSA3G9SMR","CCV1F7A8UIYH"]}]`.
+            timeout: Time to wait (in seconds) for the server to issue a response.
         """
         if not schema:
             schema = self.default_product_schema
@@ -190,6 +195,7 @@ class GraphAPI:  # pylint: disable=too-many-instance-attributes
             limit=limit,
             schema=schema.value,
             filtering=filtering,
+            timeout=timeout,
         )
 
         extracted_producs = res.json()
@@ -202,12 +208,82 @@ class GraphAPI:  # pylint: disable=too-many-instance-attributes
 
         return extracted_producs
 
+    def get_products_by_searches(
+        self,
+        queries: List[str],
+        external: bool = True,
+        force_refresh: bool = False,
+        schema: Optional[ProductSchemaName] = None,
+        timeout: Optional[int] = None,
+    ):
+        """Search for products associated with each query.
+
+        Args:
+            queries: Queries to find products for.
+            external: Whether to query external sources in order to refresh data if applicable.
+            force_refresh: Whether to force re-ingestion from external sources. Overrides
+                `external`.
+            schema: Response schema.
+            timeout: Time to wait (in seconds) for the server to issue a response.
+
+        Returns:
+            A dictionary mapping each MPN to a list of matching products.
+        """
+
+        if not queries:
+            return {}
+
+        if not schema:
+            schema = self.default_product_schema
+
+        res = httpx.post(
+            f"{self.url}/products",
+            headers=drop_none_values(
+                {
+                    "X-CLIENT-ID": self.client_id,
+                    "X-API-KEY": self.api_key,
+                }
+            ),
+            json=[
+                {
+                    "method": "GET",
+                    "relative_url": (
+                        f"?q={query}&schema={schema.value}&external={bool(external)}"
+                        f"&force_refresh={force_refresh}"
+                    ),
+                }
+                for query in queries
+            ],
+            timeout=timeout,
+        )
+
+        res.raise_for_status()
+
+        responses = res.json()
+
+        Product = schema_to_product[schema]  # pylint: disable=invalid-name
+
+        query_to_products: Dict[str, Any] = {}
+
+        for query, response in zip(queries, responses):
+            matches = []
+
+            if response["code"] == 200:
+                data = response["body"]["data"]
+
+                matches = [Product(**product_data) for product_data in data]
+
+            query_to_products[query] = matches
+
+        return query_to_products
+
     def get_products_by_ids(
         self,
         ids: List[str],
         external: Optional[bool] = True,
         force_refresh: bool = False,
         schema: Optional[ProductSchemaName] = None,
+        timeout: Optional[int] = None,
     ):
         """Get a batch of products.
 
@@ -221,6 +297,7 @@ class GraphAPI:  # pylint: disable=too-many-instance-attributes
             force_refresh: Whether to force re-ingestion from external sources. Overrides
                 `external`.
             schema: Response schema.
+            timeout: Time to wait (in seconds) for the server to issue a response.
         """
         num_requested = len(ids)
 
@@ -239,6 +316,7 @@ class GraphAPI:  # pylint: disable=too-many-instance-attributes
             schema=schema,
             filtering=[{"field": "id", "operator": "IN", "value": ids}],
             limit=BATCH_LIMIT,
+            timeout=timeout,
         )
 
         id_to_product = {p.id: p for p in extracted_products["data"]}
@@ -265,6 +343,7 @@ class GraphAPI:  # pylint: disable=too-many-instance-attributes
         after: Optional[str] = None,
         limit: Optional[int] = None,
         schema: Optional[OrgSchemaName] = None,
+        timeout: Optional[int] = None,
     ):
         """Get organizations.
 
@@ -274,6 +353,7 @@ class GraphAPI:  # pylint: disable=too-many-instance-attributes
             after: Lower page boundry, expressed as a product ID.
             limit: Restrict the results of the query to a particular number of documents.
             schema: Response schema.
+            timeout: Time to wait (in seconds) for the server to issue a response.
         """
         if not schema:
             schema = self.default_org_schema
@@ -287,6 +367,7 @@ class GraphAPI:  # pylint: disable=too-many-instance-attributes
             after=after,
             limit=limit,
             schema=schema.value,
+            timeout=timeout,
         )
 
         res_json = res.json()
@@ -304,6 +385,7 @@ class GraphAPI:  # pylint: disable=too-many-instance-attributes
         after: Optional[str] = None,
         limit: Optional[int] = None,
         schema: Optional[OrgSchemaName] = None,
+        timeout: Optional[int] = None,
     ):
         """Get suppliers.
 
@@ -313,6 +395,7 @@ class GraphAPI:  # pylint: disable=too-many-instance-attributes
             after: Lower page boundry, expressed as a product ID.
             limit: Restrict the results of the query to a particular number of documents.
             schema: Response schema.
+            timeout: Time to wait (in seconds) for the server to issue a response.
         """
         if not schema:
             schema = self.default_org_schema
@@ -326,6 +409,7 @@ class GraphAPI:  # pylint: disable=too-many-instance-attributes
             after=after,
             limit=limit,
             schema=schema.value,
+            timeout=timeout,
         )
 
         res_json = res.json()
@@ -341,6 +425,7 @@ class GraphAPI:  # pylint: disable=too-many-instance-attributes
         query: Optional[str] = None,
         limit: Optional[int] = None,
         types: Optional[str] = None,
+        timeout: Optional[int] = None,
     ) -> Dict[Literal["data"], Completion]:
         """Autocomplete organizations.
 
@@ -354,6 +439,7 @@ class GraphAPI:  # pylint: disable=too-many-instance-attributes
                 Example: "supplier" filters to suppliers.
                 Example: "supplier|manufacturer" filters to orgs that are a
                     supplier or a manufacturer.
+            timeout: Time to wait (in seconds) for the server to issue a response.
         """
 
         res = httpx.get(
@@ -371,6 +457,7 @@ class GraphAPI:  # pylint: disable=too-many-instance-attributes
                     "types": types,
                 }
             ),
+            timeout=timeout,
         )
 
         res.raise_for_status()
@@ -384,6 +471,7 @@ class GraphAPI:  # pylint: disable=too-many-instance-attributes
         external: Optional[bool] = True,
         force_refresh: bool = False,
         schema: Optional[ProductSchemaName] = None,
+        timeout: Optional[int] = None,
     ):
         """Get product.
 
@@ -396,6 +484,7 @@ class GraphAPI:  # pylint: disable=too-many-instance-attributes
             force_refresh: Whether to force re-ingestion from external sources. Overrides
                 `external`.
             schema: Response schema.
+            timeout: Time to wait (in seconds) for the server to issue a response.
         """
         if not schema:
             schema = self.default_product_schema
@@ -416,6 +505,7 @@ class GraphAPI:  # pylint: disable=too-many-instance-attributes
                     "schema": schema.value,
                 }
             ),
+            timeout=timeout,
         )
 
         res.raise_for_status()
@@ -437,6 +527,7 @@ class GraphAPI:  # pylint: disable=too-many-instance-attributes
         external: Optional[bool] = True,
         force_refresh: bool = False,
         schema: Optional[OfferSchemaName] = None,
+        timeout: Optional[int] = None,
     ):
         """Get product.
 
@@ -447,6 +538,7 @@ class GraphAPI:  # pylint: disable=too-many-instance-attributes
             force_refresh: Whether to force re-ingestion from external sources. Overrides
                 `external`.
             schema: Response schema.
+            timeout: Time to wait (in seconds) for the server to issue a response.
         """
         if not schema:
             schema = self.default_offer_schema
@@ -467,6 +559,7 @@ class GraphAPI:  # pylint: disable=too-many-instance-attributes
                     "schema": schema.value,
                 }
             ),
+            timeout=timeout,
         )
 
         res.raise_for_status()
@@ -483,6 +576,7 @@ class GraphAPI:  # pylint: disable=too-many-instance-attributes
         self,
         id: str,
         schema: Optional[OrgSchemaName] = None,
+        timeout: Optional[int] = None,
     ):
         """Get organization."""
         if not schema:
@@ -497,6 +591,7 @@ class GraphAPI:  # pylint: disable=too-many-instance-attributes
                 }
             ),
             params=drop_none_values({"schema": schema.value}),
+            timeout=timeout,
         )
 
         res.raise_for_status()
@@ -515,6 +610,7 @@ class GraphAPI:  # pylint: disable=too-many-instance-attributes
         self,
         id: str,
         schema: Optional[SupplierSchemaName] = None,
+        timeout: Optional[int] = None,
     ):
         """Get supplier."""
         if not schema:
@@ -529,6 +625,7 @@ class GraphAPI:  # pylint: disable=too-many-instance-attributes
                 }
             ),
             params=drop_none_values({"schema": schema.value}),
+            timeout=timeout,
         )
 
         res.raise_for_status()
